/*
 * Author: Kipp Cannon <kipp@gravity.phys.uwm.edu>
 * Revision: $Id: TimeSeries.m4,v 1.1 2005/01/28 23:38:06 kipp Exp $
 */

#include <string.h>
#include <lal/Date.h>
#include <lal/LALDatatypes.h>
#include <lal/LALStdlib.h>
#include <lal/LALStatusMacros.h>
#include <lal/LALErrno.h>
#include <lal/LALError.h>
#include <lal/Sequence.h>
#include <lal/TimeSeries.h>

NRCSID(TIMESERIESC, "$Id: TimeSeries.m4,v 1.1 2005/01/28 23:38:06 kipp Exp $");




void XLALDestroyREAL4TimeSeries (
	REAL4TimeSeries *series
)
{
	if(series)
		XLALDestroyREAL4Sequence (series->data);
	LALFree(series);
}


void LALDestroyREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyREAL4TimeSeries", TIMESERIESC);
	XLALDestroyREAL4TimeSeries (series);
	RETURN(status);
}


REAL4TimeSeries *XLALCreateREAL4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	REAL4TimeSeries *new;
	REAL4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateREAL4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateREAL4TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateREAL4TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


REAL4TimeSeries *XLALCutREAL4TimeSeries (
	REAL4TimeSeries *series,
	size_t first,
	size_t length
)
{
	REAL4TimeSeries *new;
	REAL4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutREAL4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries **output,
	REAL4TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutREAL4TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutREAL4TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeREAL4TimeSeries (
	REAL4TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeREAL4Sequence (series->data, first, length));
}


void LALResizeREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeREAL4TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeREAL4TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkREAL4TimeSeries (
	REAL4TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeREAL4TimeSeries (series, first, length));
}


void LALShrinkREAL4TimeSeries (
	LALStatus *status,
	REAL4TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkREAL4TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkREAL4TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyREAL8TimeSeries (
	REAL8TimeSeries *series
)
{
	if(series)
		XLALDestroyREAL8Sequence (series->data);
	LALFree(series);
}


void LALDestroyREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyREAL8TimeSeries", TIMESERIESC);
	XLALDestroyREAL8TimeSeries (series);
	RETURN(status);
}


REAL8TimeSeries *XLALCreateREAL8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	REAL8TimeSeries *new;
	REAL8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateREAL8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateREAL8TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateREAL8TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


REAL8TimeSeries *XLALCutREAL8TimeSeries (
	REAL8TimeSeries *series,
	size_t first,
	size_t length
)
{
	REAL8TimeSeries *new;
	REAL8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutREAL8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries **output,
	REAL8TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutREAL8TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutREAL8TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeREAL8TimeSeries (
	REAL8TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeREAL8Sequence (series->data, first, length));
}


void LALResizeREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeREAL8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeREAL8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkREAL8TimeSeries (
	REAL8TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeREAL8TimeSeries (series, first, length));
}


void LALShrinkREAL8TimeSeries (
	LALStatus *status,
	REAL8TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkREAL8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkREAL8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX8Sequence (series->data);
	LALFree(series);
}


void LALDestroyCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyCOMPLEX8TimeSeries", TIMESERIESC);
	XLALDestroyCOMPLEX8TimeSeries (series);
	RETURN(status);
}


COMPLEX8TimeSeries *XLALCreateCOMPLEX8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	COMPLEX8TimeSeries *new;
	COMPLEX8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateCOMPLEX8TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateCOMPLEX8TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


COMPLEX8TimeSeries *XLALCutCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
)
{
	COMPLEX8TimeSeries *new;
	COMPLEX8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries **output,
	COMPLEX8TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutCOMPLEX8TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutCOMPLEX8TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeCOMPLEX8Sequence (series->data, first, length));
}


void LALResizeCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeCOMPLEX8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeCOMPLEX8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeCOMPLEX8TimeSeries (series, first, length));
}


void LALShrinkCOMPLEX8TimeSeries (
	LALStatus *status,
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkCOMPLEX8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkCOMPLEX8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX16Sequence (series->data);
	LALFree(series);
}


void LALDestroyCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyCOMPLEX16TimeSeries", TIMESERIESC);
	XLALDestroyCOMPLEX16TimeSeries (series);
	RETURN(status);
}


COMPLEX16TimeSeries *XLALCreateCOMPLEX16TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	COMPLEX16TimeSeries *new;
	COMPLEX16Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX16Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateCOMPLEX16TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateCOMPLEX16TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


COMPLEX16TimeSeries *XLALCutCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
)
{
	COMPLEX16TimeSeries *new;
	COMPLEX16Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX16Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries **output,
	COMPLEX16TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutCOMPLEX16TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutCOMPLEX16TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeCOMPLEX16Sequence (series->data, first, length));
}


void LALResizeCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeCOMPLEX16TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeCOMPLEX16TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeCOMPLEX16TimeSeries (series, first, length));
}


void LALShrinkCOMPLEX16TimeSeries (
	LALStatus *status,
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkCOMPLEX16TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkCOMPLEX16TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyINT2TimeSeries (
	INT2TimeSeries *series
)
{
	if(series)
		XLALDestroyINT2Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyINT2TimeSeries", TIMESERIESC);
	XLALDestroyINT2TimeSeries (series);
	RETURN(status);
}


INT2TimeSeries *XLALCreateINT2TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	INT2TimeSeries *new;
	INT2Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT2Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT2Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT2TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT2TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


INT2TimeSeries *XLALCutINT2TimeSeries (
	INT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	INT2TimeSeries *new;
	INT2Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT2Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries **output,
	INT2TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutINT2TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutINT2TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeINT2TimeSeries (
	INT2TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeINT2Sequence (series->data, first, length));
}


void LALResizeINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeINT2TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeINT2TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkINT2TimeSeries (
	INT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT2TimeSeries (series, first, length));
}


void LALShrinkINT2TimeSeries (
	LALStatus *status,
	INT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT2TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT2TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyUINT2TimeSeries (
	UINT2TimeSeries *series
)
{
	if(series)
		XLALDestroyUINT2Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT2TimeSeries", TIMESERIESC);
	XLALDestroyUINT2TimeSeries (series);
	RETURN(status);
}


UINT2TimeSeries *XLALCreateUINT2TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	UINT2TimeSeries *new;
	UINT2Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT2Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT2TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT2TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


UINT2TimeSeries *XLALCutUINT2TimeSeries (
	UINT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	UINT2TimeSeries *new;
	UINT2Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries **output,
	UINT2TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutUINT2TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutUINT2TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeUINT2TimeSeries (
	UINT2TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeUINT2Sequence (series->data, first, length));
}


void LALResizeUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeUINT2TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeUINT2TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkUINT2TimeSeries (
	UINT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT2TimeSeries (series, first, length));
}


void LALShrinkUINT2TimeSeries (
	LALStatus *status,
	UINT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT2TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT2TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyINT4TimeSeries (
	INT4TimeSeries *series
)
{
	if(series)
		XLALDestroyINT4Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyINT4TimeSeries", TIMESERIESC);
	XLALDestroyINT4TimeSeries (series);
	RETURN(status);
}


INT4TimeSeries *XLALCreateINT4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	INT4TimeSeries *new;
	INT4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT4Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT4TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT4TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


INT4TimeSeries *XLALCutINT4TimeSeries (
	INT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	INT4TimeSeries *new;
	INT4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT4Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries **output,
	INT4TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutINT4TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutINT4TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeINT4TimeSeries (
	INT4TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeINT4Sequence (series->data, first, length));
}


void LALResizeINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeINT4TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeINT4TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkINT4TimeSeries (
	INT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT4TimeSeries (series, first, length));
}


void LALShrinkINT4TimeSeries (
	LALStatus *status,
	INT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT4TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT4TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyUINT4TimeSeries (
	UINT4TimeSeries *series
)
{
	if(series)
		XLALDestroyUINT4Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT4TimeSeries", TIMESERIESC);
	XLALDestroyUINT4TimeSeries (series);
	RETURN(status);
}


UINT4TimeSeries *XLALCreateUINT4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	UINT4TimeSeries *new;
	UINT4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT4TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT4TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


UINT4TimeSeries *XLALCutUINT4TimeSeries (
	UINT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	UINT4TimeSeries *new;
	UINT4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries **output,
	UINT4TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutUINT4TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutUINT4TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeUINT4TimeSeries (
	UINT4TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeUINT4Sequence (series->data, first, length));
}


void LALResizeUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeUINT4TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeUINT4TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkUINT4TimeSeries (
	UINT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT4TimeSeries (series, first, length));
}


void LALShrinkUINT4TimeSeries (
	LALStatus *status,
	UINT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT4TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT4TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyINT8TimeSeries (
	INT8TimeSeries *series
)
{
	if(series)
		XLALDestroyINT8Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyINT8TimeSeries", TIMESERIESC);
	XLALDestroyINT8TimeSeries (series);
	RETURN(status);
}


INT8TimeSeries *XLALCreateINT8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	INT8TimeSeries *new;
	INT8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT8Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT8TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT8TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


INT8TimeSeries *XLALCutINT8TimeSeries (
	INT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	INT8TimeSeries *new;
	INT8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries **output,
	INT8TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutINT8TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutINT8TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeINT8TimeSeries (
	INT8TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeINT8Sequence (series->data, first, length));
}


void LALResizeINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeINT8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeINT8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkINT8TimeSeries (
	INT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeINT8TimeSeries (series, first, length));
}


void LALShrinkINT8TimeSeries (
	LALStatus *status,
	INT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}






void XLALDestroyUINT8TimeSeries (
	UINT8TimeSeries *series
)
{
	if(series)
		XLALDestroyUINT8Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT8TimeSeries", TIMESERIESC);
	XLALDestroyUINT8TimeSeries (series);
	RETURN(status);
}


UINT8TimeSeries *XLALCreateUINT8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	UINT8TimeSeries *new;
	UINT8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		return(NULL);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaT,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT8TimeSeries", TIMESERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT8TimeSeries (name, &epoch, f0, deltaT, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


UINT8TimeSeries *XLALCutUINT8TimeSeries (
	UINT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	UINT8TimeSeries *new;
	UINT8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	XLALAddFloatToGPS(&new->epoch, first * new->deltaT);

	return(new);
}


void LALCutUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries **output,
	UINT8TimeSeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutUINT8TimeSeries", TIMESERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutUINT8TimeSeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALResizeUINT8TimeSeries (
	UINT8TimeSeries *series,
	int first,
	size_t length
)
{
	if(!series)
		return(0);

	XLALAddFloatToGPS(&series->epoch, first * series->deltaT);
	return(XLALResizeUINT8Sequence (series->data, first, length));
}


void LALResizeUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries *series,
	int first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALResizeUINT8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	new_length = XLALResizeUINT8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


size_t XLALShrinkUINT8TimeSeries (
	UINT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	return(XLALResizeUINT8TimeSeries (series, first, length));
}


void LALShrinkUINT8TimeSeries (
	LALStatus *status,
	UINT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT8TimeSeries", TIMESERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT8TimeSeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}


